import sys
import os
import json
import pickle


def Repeat(x):
    _size = len(x)
    repeated = []
    for i in range(_size):
        k = i + 1
        for j in range(k, _size):
            if x[i] == x[j] and x[i] not in repeated:
                repeated.append(x[i])
    return repeated


def testt(A, qqqq=True, sss=True):
    aa = []
    for i in A:
        aa.append(i['id'])
    if qqqq:
        print("MAX {}".format(max(aa)))
        print("MIN {}".format(min(aa)))
    if sss:
        return Repeat(aa)
    return aa


def mm_red(a1, aa):
    with open(a1) as json_file:
        d1 = json.load(json_file)
    ind = []
    res = []
    k = []
    ann = []
    a = d1['categories'].copy()
    print("Initial checks:")
    l1 = [i['name'] for i in d1['categories']]

    def chch(aac):
        res = []
        [res.append(i) for i in aac if i not in res]
        return res

    print("\tPrevious labels: {}".format(l1))
    if any(x not in l1 for x in aa):
        print("\tLabels not found: {}".format([i for i in aa if i not in l1]))
        return {}
    print("\tChecking Images...")
    if len(testt(d1['images'], qqqq=False)) != 0:
        print("\t\tImages section error: {} duplicate ids".format(len(testt(d1['images'], qqqq=False))))
        print("\t\t\tChecking for solution...")
        d1['images'] = chch(d1['images'])
        if len(testt(d1['images'], qqqq=False)) != 0:
            print("\t\t\tImages couldn't fix: {} duplicate id are still present".format(
                len(testt(d1['images'], qqqq=False))))
            return {}
        print("\t\t\tImages duplicates fixed succesfully!")
    print("\tChecking Annotations...")
    if len(testt(d1['annotations'], qqqq=False)) != 0:
        print("\t\tAnnotations section error: {} duplicate ids".format(len(testt(d1['annotations'], qqqq=False))))
        print("\t\t\tChecking for solution...")
        d1['annotations'] = chch(d1['annotations'])
        if len(testt(d1['annotations'], qqqq=False)) != 0:
            print("\t\t\tAnnotations couldn't fix: {} duplicate id are still present".format(
                len(testt(d1['annotations'], qqqq=False))))
            return {}
        print("\t\t\tAnnotations duplicates fixed succesfully!")
    print("Stage 1...")
    [ind.append(i) for i, elem in enumerate(a) if any(x in elem.values() for x in aa)]
    ind.sort(reverse=True)
    [k.append(d1['categories'][i]['id']) for i in ind]
    for i in ind: del a[i]
    [ann.append(j) for i, j in enumerate(d1['annotations']) if d1['annotations'][i]['category_id'] not in k]
    print("Stage 2...")
    d1['categories'] = a
    res2 = []
    res3 = []
    [res2.append(i['image_id']) for i in ann if i not in res]
    [res3.append(i['file_name']) for i in d1['images'] if i['id'] in res2]
    asd = input(
        'Please enter images filepath to clean or "NA" to return deleted images files as text file ("temp_mm_delete.txt"):')
    if asd != "NA":
        [os.remove[os.path.join(asd, i)] for i in res3]
    else:
        with open("temp_mm_delete.txt", "wb") as fp:
            pickle.dump(res3, fp)
    res3 = []
    [res3.append(i) for i in d1['images'] if i['id'] in res2]
    d1['images'] = res3
    print("Annotations Summary: Initial {}  Final {} Removed {}".format(len(d1['annotations']), len(ann),
                                                                        len(d1['annotations']) - len(ann)))
    d1['annotations'] = ann
    return d1


if __name__ == '__main__':
    if len(sys.argv) <= 2:
        print('2 auguments are need.')
        print('__________________________________________________________________________')
        print('Usage: %s INPUT_JSON.json OUTPU_JSON.json Label1 Label2...' % (sys.argv[0]))
        print('__________________________________________________________________________')
        exit(1)
    jsona = json.dumps(mm_red(sys.argv[1], sys.argv[3:]))
    assert json.dumps({}) != '{}'
    print("Saving..")
    with open(sys.argv[2], "w") as f:
        f.write(jsona)
    print("\n\nThanks for using our service :) !!")



import argparse
import sys
import os
import json
from tqdm import tqdm


def dict_compare(d1, d2):
    d1_keys = set(d1.keys())
    d2_keys = set(d2.keys())
    shared_keys = d1_keys.intersection(d2_keys)
    added = d1_keys - d2_keys
    removed = d2_keys - d1_keys
    modified = {o: (d1[o], d2[o]) for o in shared_keys if d1[o] != d2[o]}
    same = set(o for o in shared_keys if d1[o] == d2[o])
    return added, removed, modified, same


def Repeat(x):
    _size = len(x)
    repeated = []
    for i in range(_size):
        k = i + 1
        for j in range(k, _size):
            if x[i] == x[j] and x[i] not in repeated:
                repeated.append(x[i])
    return repeated


def testt(A):
    aa = []
    for i in A:
        aa.append(i['id'])
    print("MAX {}".format(max(aa)))
    print("MIN {}".format(min(aa)))
    return Repeat(aa)


def combine(tt1, tt2, output_file):
    """ Combine two COCO annoatated files and save them into new file
    :param tt1: 1st COCO file path
    :param tt2: 2nd COCO file path
    :param output_file: output file path
    """
    with open(tt1) as json_file:
        d1 = json.load(json_file)
    with open(tt2) as json_file:
        d2 = json.load(json_file)
    b1 = {}
    for i, j in enumerate(d1['images']):
        b1[d1['images'][i]['id']] = i

    temp = [cc['file_name'] for cc in d1['images']]
    temp2 = [cc['file_name'] for cc in d2['images']]
    for i in temp:
        assert not (i in temp2), "Duplicate filenames detected between the two files! @" + i

    # Check if both files have the categories dict using only the value and id to compare
    d1_categories_names = {c['name']: c['id'] for c in d1['categories']}
    d2_categories_names = {c['name']: c['id'] for c in d2['categories']}

    for c in d1_categories_names:
        # Check if the category name exists in the second file
        if c in d2_categories_names:
            # Check if the category id is the same
            if d1_categories_names[c] != d2_categories_names[c]:
                assert False, 'Category name: {}, id: {} in file 1 and {} in file 2'.format(c, d1_categories_names[c],
                                                                                            d2_categories_names[c])
        else:
            assert False, 'Category name: {} in file 1 does not exist in file 2'.format(c)

    for c in d2_categories_names:
        if c in d1_categories_names:
            if d1_categories_names[c] != d2_categories_names[c]:
                assert False, 'Category name: {}, id: {} in file 1 and {} in file 2'.format(c, d1_categories_names[c],
                                                                                            d2_categories_names[c])
        else:
            assert False, 'Category name: {} in file 2 does not exist in file 1'.format(c)

    files_check_classes = {}
    for i, j in enumerate(d1['images']):
        for ii, jj in enumerate(d1['annotations']):
            if jj['image_id'] == j['id']:
                try:
                    files_check_classes[j['file_name']].append(jj['category_id'])
                except:
                    files_check_classes[j['file_name']] = [jj['category_id']]

    for i, j in enumerate(d2['images']):
        for ii, jj in enumerate(d2['annotations']):
            if jj['image_id'] == j['id']:
                try:
                    files_check_classes[j['file_name']].append(jj['category_id'])
                except:
                    files_check_classes[j['file_name']] = [jj['category_id']]

    b2 = {}
    for i, j in enumerate(d2['images']):
        b2[d2['images'][i]['id']] = i + max(b1) + 1

    # Reset File 1 and 2 images ids
    for i, j in enumerate(d1['images']):
        d1['images'][i]['id'] = b1[d1['images'][i]['id']]
    for i, j in enumerate(d2['images']):
        d2['images'][i]['id'] = b2[d2['images'][i]['id']]

    # Reset File 1 and 2 annotations ids
    b3 = {}
    for i, j in enumerate(d1['annotations']):
        b3[d1['annotations'][i]['id']] = i
    b4 = {}
    for i, j in enumerate(d2['annotations']):
        b4[d2['annotations'][i]['id']] = max(b3) + i + 1

    for i, j in enumerate(d1['annotations']):
        d1['annotations'][i]['id'] = b3[d1['annotations'][i]['id']]
        d1['annotations'][i]['image_id'] = b1[d1['annotations'][i]['image_id']]
    for i, j in enumerate(d2['annotations']):
        d2['annotations'][i]['id'] = b4[d2['annotations'][i]['id']]
        d2['annotations'][i]['image_id'] = b2[d2['annotations'][i]['image_id']]

    files_check_classes_temp = {}
    pbar = tqdm(total=len(d1['images']) + len(d2['images']))
    for i, j in enumerate(d1['images']):
        for ii, jj in enumerate(d1['annotations']):
            if jj['image_id'] == j['id']:
                try:
                    files_check_classes_temp[j['file_name']].append(jj['category_id'])
                except:
                    pbar.update(1)
                    files_check_classes_temp[j['file_name']] = [jj['category_id']]

    for i, j in enumerate(d2['images']):
        for ii, jj in enumerate(d2['annotations']):
            if jj['image_id'] == j['id']:
                try:
                    files_check_classes_temp[j['file_name']].append(jj['category_id'])
                except:
                    pbar.update(1)
                    files_check_classes_temp[j['file_name']] = [jj['category_id']]
    pbar.close()
    added, removed, modified, same = dict_compare(files_check_classes, files_check_classes_temp)
    assert (len(added) == 0 and len(removed) == 0 and len(
        modified) == 0), "filenames detected before merging error: " + len(added) + " filenames added " + len(
        removed) + " filenames removed " + len(modified) + " filenames' classes modified " + len(
        same) + " filenames entries reserved"

    test = d1.copy()
    for i in d2['images']:
        test['images'].append(i)
    for i in d2['annotations']:
        test['annotations'].append(i)
    test['categories'] = d2['categories']
    files_check_classes_temp = {}
    pbar = tqdm(total=len(test['images']))
    for i, j in enumerate(test['images']):
        for ii, jj in enumerate(test['annotations']):
            if jj['image_id'] == j['id']:
                try:
                    files_check_classes_temp[j['file_name']].append(jj['category_id'])
                except:
                    pbar.update(1)
                    files_check_classes_temp[j['file_name']] = [jj['category_id']]

    pbar.close()
    added, removed, modified, same = dict_compare(files_check_classes, files_check_classes_temp)
    assert (len(added) == 0 and len(removed) == 0 and len(
        modified) == 0), "filenames detected after merging error: " + len(added) + " filenames added " + len(
        removed) + " filenames removed " + len(modified) + " filenames' classes modified " + len(
        same) + " filenames entries reserved"

    with open(output_file, 'w') as f:
        json.dump(test, f)


def run():
    parser = argparse.ArgumentParser(description='Merge two annotation files to one file')
    parser.add_argument('--src', required=True, type=str, nargs=2,
                        help='Path of two annotation files to merge')
    parser.add_argument('--out', required=True, type=str,
                        help='Path of the output annotation file')

    args = parser.parse_args()

    combine(args.src[0], args.src[1], args.out)
    print("\n\nSuccessfully merged the two files ({} , {}) into {}".format(args.src[0], args.src[1], args.out))


if __name__ == '__main__':
    run()



import os

import setuptools

with open(os.path.join(os.path.dirname(__file__), "README.md")) as readme:
    README = readme.read()

setuptools.setup(
    name="COCO_merger",
    version="0.0.1",
    author="Mohamad Mansour & Tristan Cotte",
    author_email="moemansour03@gmail.com",
    description="Python package which aims to merge 2 COCO .json files",
    long_description="This package is intended for data scientist who wants to merge several COCO datasets before "
                     "training a new model",
    long_description_content_type="text/markdown",
    url="https://github.com/mohamadmansourX/Merge_COCO_FILES",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)
